@@ -1156,6 +1156,29 @@ ggit_diff_find_flags_get_type (void)
 	return the_type;
 }
 
+GType
+ggit_diff_format_email_flags_get_type (void)
+{
+	static GType the_type = 0;
+	
+	if (the_type == 0)
+	{
+		static const GFlagsValue values[] = {
+			{ GGIT_DIFF_FORMAT_EMAIL_NONE,
+			  "GGIT_DIFF_FORMAT_EMAIL_NONE",
+			  "none" },
+			{ GGIT_DIFF_FORMAT_EMAIL_EXCLUDE_SUBJECT_PATCH_MARKER,
+			  "GGIT_DIFF_FORMAT_EMAIL_EXCLUDE_SUBJECT_PATCH_MARKER",
+			  "exclude-subject-patch-marker" },
+			{ 0, NULL, NULL }
+		};
+		the_type = g_flags_register_static (
+				g_intern_static_string ("GgitDiffFormatEmailFlags"),
+				values);
+	}
+	return the_type;
+}
+
 
 /* Generated data ends here */
 
