@@ -84,6 +84,7 @@ _ggit_index_entry_new (const gchar *path,
 	entry = g_slice_new0 (git_index_entry);
 
 	ret = ggit_index_entry_wrap (entry, TRUE);
+	entry->mode = GIT_FILEMODE_BLOB;
 
 	ggit_index_entry_set_path (ret, path);
 	ggit_index_entry_set_id (ret, id);
@@ -126,7 +127,7 @@ ggit_index_entry_unref (GgitIndexEntry *entry)
 	{
 		if (entry->owned)
 		{
-			g_free (entry->entry->path);
+			g_free ((gchar *)entry->entry->path);
 			g_slice_free (git_index_entry, entry->entry);
 		}
 
@@ -656,7 +657,7 @@ ggit_index_entry_set_path (GgitIndexEntry *entry,
 
 	if (entry->entry->path)
 	{
-		g_free (entry->entry->path);
+		g_free ((gchar *)entry->entry->path);
 		entry->entry->path = NULL;
 	}
 
@@ -666,6 +667,27 @@ ggit_index_entry_set_path (GgitIndexEntry *entry,
 	}
 }
 
+static git_filemode_t
+file_info_to_filemode (GFileInfo *info)
+{
+	git_filemode_t ret = 0;
+
+	if (g_file_info_get_is_symlink (info))
+	{
+		ret |= GIT_FILEMODE_LINK;
+	}
+	else if (g_file_info_get_attribute_boolean (info, G_FILE_ATTRIBUTE_ACCESS_CAN_EXECUTE))
+	{
+		ret |= GIT_FILEMODE_BLOB_EXECUTABLE;
+	}
+	else
+	{
+		ret |= GIT_FILEMODE_BLOB;
+	}
+
+	return ret;
+}
+
 /**
  * ggit_index_entry_stat:
  * @entry: a #GgitIndexEntry.
@@ -697,7 +719,7 @@ ggit_index_entry_stat (GgitIndexEntry  *entry,
 	                          G_FILE_ATTRIBUTE_TIME_CREATED_USEC ","
 	                          G_FILE_ATTRIBUTE_UNIX_DEVICE ","
 	                          G_FILE_ATTRIBUTE_UNIX_INODE ","
-	                          G_FILE_ATTRIBUTE_UNIX_MODE ","
+	                          G_FILE_ATTRIBUTE_ACCESS_CAN_EXECUTE ","
 	                          G_FILE_ATTRIBUTE_UNIX_UID ","
 	                          G_FILE_ATTRIBUTE_UNIX_GID,
 	                          G_FILE_QUERY_INFO_NOFOLLOW_SYMLINKS,
@@ -735,9 +757,7 @@ ggit_index_entry_stat (GgitIndexEntry  *entry,
 		g_file_info_get_attribute_uint64 (info,
 		                                  G_FILE_ATTRIBUTE_UNIX_INODE);
 
-	entry->entry->mode =
-		g_file_info_get_attribute_uint32 (info,
-		                                  G_FILE_ATTRIBUTE_UNIX_MODE);
+	entry->entry->mode = file_info_to_filemode (info);
 
 	entry->entry->uid =
 		g_file_info_get_attribute_uint32 (info,
