@@ -36,6 +36,8 @@
 #include "ggit-branch-enumerator.h"
 #include "ggit-blame.h"
 #include "ggit-blame-options.h"
+#include "ggit-commit.h"
+#include "ggit-revert-options.h"
 
 #define GGIT_REPOSITORY_GET_PRIVATE(object)(G_TYPE_INSTANCE_GET_PRIVATE((object), GGIT_TYPE_REPOSITORY, GgitRepositoryPrivate))
 
@@ -1330,6 +1332,55 @@ ggit_repository_create_tag_lightweight (GgitRepository   *repository,
 	return _ggit_oid_wrap (&oid);
 }
 
+/**
+ * ggit_repository_create_tag_annotation:
+ * @repository: a #GgitRepository.
+ * @tag_name: the name of the tag.
+ * @target: a #GgitObject.
+ * @signature: a #GgitObject.
+ * @message: the tag message.
+ * @error: a #GError for error reporting, or %NULL.
+ *
+ * Creates a new annotated tag.
+ *
+ * Returns: (transfer full) (allow-none): the id to which the tag points, or
+ *                                        %NULL in case of an error.
+ *
+ **/
+GgitOId *
+ggit_repository_create_tag_annotation (GgitRepository  *repository,
+                                       const gchar     *tag_name,
+                                       GgitObject      *target,
+                                       GgitSignature   *signature,
+                                       const gchar     *message,
+                                       GError         **error)
+{
+	git_oid oid;
+	gint ret;
+
+	g_return_val_if_fail (GGIT_IS_REPOSITORY (repository), FALSE);
+	g_return_val_if_fail (tag_name != NULL, FALSE);
+	g_return_val_if_fail (GGIT_IS_OBJECT (target), FALSE);
+	g_return_val_if_fail (GGIT_IS_SIGNATURE (signature), FALSE);
+	g_return_val_if_fail (message != NULL, FALSE);
+	g_return_val_if_fail (error == NULL || *error == NULL, FALSE);
+
+	ret = git_tag_annotation_create (&oid,
+	                                 _ggit_native_get (repository),
+	                                 tag_name,
+	                                 _ggit_native_get (target),
+	                                 _ggit_native_get (signature),
+	                                 message);
+
+	if (ret != GIT_OK)
+	{
+		_ggit_error_set (error, ret);
+		return NULL;
+	}
+
+	return _ggit_oid_wrap (&oid);
+}
+
 /**
  * ggit_repository_list_tags:
  * @repository: a #GgitRepository.
@@ -1366,6 +1417,146 @@ ggit_repository_list_tags (GgitRepository  *repository,
 	return tags;
 }
 
+/**
+ * ggit_repository_list_tags_match:
+ * @repository: a #GgitRepository.
+ * @pattern: (allow-none): a pattern to match.
+ * @error: a #GError for error reporting, or %NULL.
+ *
+ * Fill a list with all the tags in the @repository matching the provided
+ * pattern. The pattern can use standard fnmatch syntax.
+ *
+ * Returns: (transfer full) (allow-none): a list with matching tags in @repository.
+ **/
+gchar **
+ggit_repository_list_tags_match (GgitRepository  *repository,
+                                 const gchar     *pattern,
+                                 GError         **error)
+{
+	gint ret;
+	git_strarray tag_names;
+	gchar **tags;
+
+	g_return_val_if_fail (GGIT_IS_REPOSITORY (repository), FALSE);
+	g_return_val_if_fail (error == NULL || *error == NULL, FALSE);
+
+	ret = git_tag_list_match (&tag_names,
+	                          pattern ? pattern : "",
+	                          _ggit_native_get (repository));
+
+	if (ret != GIT_OK)
+	{
+		_ggit_error_set (error, ret);
+		tags = NULL;
+	}
+	else
+	{
+		tags = ggit_utils_get_str_array_from_git_strarray (&tag_names);
+	}
+
+	return tags;
+}
+
+/**
+ * ggit_repository_delete_tag:
+ * @repository: a #GgitRepository.
+ * @name: the name of the tag.
+ * @error: a #GError.
+ *
+ * Delete an existing tag reference by name.
+ *
+ * Returns: %TRUE if the tag was deleted successfully, %FALSE otherwise.
+ **/
+gboolean
+ggit_repository_delete_tag (GgitRepository  *repository,
+                            gchar const     *name,
+                            GError         **error)
+{
+	gint ret;
+
+	g_return_val_if_fail (GGIT_IS_REPOSITORY (repository), FALSE);
+	g_return_val_if_fail (name != NULL, FALSE);
+	g_return_val_if_fail (error == NULL || *error == NULL, FALSE);
+
+	ret = git_tag_delete (_ggit_native_get (repository), name);
+
+	if (ret != GIT_OK)
+	{
+		_ggit_error_set (error, ret);
+		return FALSE;
+	}
+	else
+	{
+		return TRUE;
+	}
+}
+
+typedef struct
+{
+	gpointer user_data;
+
+	GgitTagCallback callback;
+} TagCallbackWrapperData;
+
+static gint
+tag_callback_wrapper (const gchar *name,
+                      git_oid     *tag_oid,
+                      gpointer     user_data)
+{
+	TagCallbackWrapperData *wrapper_data = (TagCallbackWrapperData *)user_data;
+	GgitOId *oid;
+	gint ret;
+
+	oid = _ggit_oid_wrap (tag_oid);
+
+	ret = wrapper_data->callback (name, oid, wrapper_data->user_data);
+	ggit_oid_free (oid);
+
+	return ret;
+}
+
+/**
+ * ggit_repository_tag_foreach:
+ * @repository: a #GgitRepository.
+ * @callback: (scope call): a #GgitTagCallback.
+ * @user_data: callback user data.
+ * @error: a #GError for error reporting, or %NULL.
+ *
+ * Get all the tags in the repository and run the provided callback on each. If
+ * the callback returns something other than 0, the iteration will stop and
+ * @error will be set.
+ *
+ * Returns: %TRUE if there was no error, %FALSE otherwise
+ **/
+gboolean
+ggit_repository_tag_foreach (GgitRepository   *repository,
+                             GgitTagCallback   callback,
+                             gpointer          user_data,
+                             GError          **error)
+{
+	TagCallbackWrapperData wrapper_data;
+	gint ret;
+
+	g_return_val_if_fail (GGIT_IS_REPOSITORY (repository), FALSE);
+	g_return_val_if_fail (callback != NULL, FALSE);
+	g_return_val_if_fail (error == NULL || *error == NULL, FALSE);
+
+	wrapper_data.user_data = user_data;
+	wrapper_data.callback = callback;
+
+	ret = git_tag_foreach (_ggit_native_get (repository),
+	                       tag_callback_wrapper,
+	                       &wrapper_data);
+
+	if (ret != GIT_OK)
+	{
+		_ggit_error_set (error, ret);
+		return FALSE;
+	}
+
+	return TRUE;
+}
+
 /**
  * ggit_repository_create_branch:
  * @repository: a #GgitRepository.
@@ -1421,9 +1612,9 @@ ggit_repository_create_branch (GgitRepository   *repository,
 }
 
 /**
- * ggit_repository_branches_foreach:
+ * ggit_repository_enumerate_branches:
  * @repository: a #GgitRepository.
- * @list_type: a GgitBranchType.
+ * @list_type: a #GgitBranchType.
  * @error: a #GError for error reporting, or %NULL.
  *
  * Get a branch enumerator to enumerate over all branches of the specified
@@ -1735,8 +1926,8 @@ ggit_repository_reset (GgitRepository  *repository,
 
 	g_return_if_fail (GGIT_IS_REPOSITORY (repository));
 	g_return_if_fail (GGIT_IS_OBJECT (target));
-	g_return_val_if_fail (GGIT_IS_SIGNATURE (signature), FALSE);
-	g_return_val_if_fail (log_message != NULL, FALSE);
+	g_return_if_fail (GGIT_IS_SIGNATURE (signature));
+	g_return_if_fail (log_message != NULL);
 	g_return_if_fail (error == NULL || *error == NULL);
 
 	ret = git_reset (_ggit_native_get (repository),
@@ -1751,6 +1942,47 @@ ggit_repository_reset (GgitRepository  *repository,
 	}
 }
 
+/**
+ * ggit_repository_reset_default:
+ * @repository: a #GgitRepository.
+ * @target: (allow-none): the target #GgitObject which is a commit or a tag.
+ * @pathspecs: a list of file paths to reset.
+ * @error: a #GError for error reporting or %NULL.
+ *
+ * Update some entries in the index from the target commit tree. The scope of
+ * the updated entries is determined by the paths specified in @pathspecs.
+ * Passing %NULL in @target will result in removing entries in the index
+ * matching the provided pathspecs.
+ *
+ **/
+void
+ggit_repository_reset_default (GgitRepository       *repository,
+                               GgitObject           *target,
+                               const gchar * const  *pathspecs,
+                               GError              **error)
+{
+	gint ret;
+	git_strarray pp;
+
+	g_return_if_fail (GGIT_IS_REPOSITORY (repository));
+	g_return_if_fail (target == NULL || GGIT_IS_OBJECT (target));
+	g_return_if_fail (pathspecs != NULL && *pathspecs != NULL);
+	g_return_if_fail (error == NULL || *error == NULL);
+
+	ggit_utils_get_git_strarray_from_str_array (pathspecs, &pp);
+
+	ret = git_reset_default (_ggit_native_get (repository),
+	                         _ggit_native_get (target),
+	                         &pp);
+
+	git_strarray_free (&pp);
+
+	if (ret != GIT_OK)
+	{
+		_ggit_error_set (error, ret);
+	}
+}
+
 /**
  * ggit_repository_save_stash:
  * @repository: a #GgitRepository.
@@ -1932,6 +2164,51 @@ ggit_repository_get_ahead_behind (GgitRepository  *repository,
 	}
 }
 
+/**
+ * ggit_repository_get_descendant_of:
+ * @repository: a #GgitRepository.
+ * @commit: the commit.
+ * @ancestor: the ancestor.
+ * @error: a #GError for error reporting, or %NULL.
+ *
+ * Check whether @com mit is a descendant of @ancestor. Note that if this
+ * function returns %FALSE, an error might have occurred. If so, @error will
+ * be set appropriately.
+ *
+ * Returns: %TRUE if @commit is a descendant of @ancestor, or %FALSE otherwise.
+ *
+ */
+gboolean
+ggit_repository_get_descendant_of (GgitRepository  *repository,
+                                   GgitOId         *commit,
+                                   GgitOId         *ancestor,
+                                   GError         **error)
+{
+	gint ret;
+
+	g_return_val_if_fail (GGIT_IS_REPOSITORY (repository), FALSE);
+	g_return_val_if_fail (commit != NULL, FALSE);
+	g_return_val_if_fail (ancestor != NULL, FALSE);
+	g_return_val_if_fail (error == NULL || *error == NULL, FALSE);
+
+	ret = git_graph_descendant_of (_ggit_native_get (repository),
+	                               _ggit_oid_get_oid (commit),
+	                               _ggit_oid_get_oid (ancestor));
+
+	if (ret == 1)
+	{
+		return TRUE;
+	}
+
+	if (ret != GIT_OK)
+	{
+		_ggit_error_set (error, ret);
+
+	}
+
+	return FALSE;
+}
+
 /**
  * ggit_repository_create_blob:
  * @repository: a #GgitRepository.
@@ -2469,4 +2746,162 @@ ggit_repository_get_attribute (GgitRepository           *repository,
 	return value;
 }
 
+/**
+ * ggit_repository_checkout_head:
+ * @repository: a #GgitRepository.
+ * @options: (allow-none): a #GgitCheckoutOptions or %NULL.
+ * @error: a #GError for error reporting or %NULL.
+ *
+ * Update files in the working tree to reflect the contents of current HEAD. If
+ * @options is %NULL, then the default checkout options will be used.
+ *
+ * If the checkout was not successfull, then @error will be set.
+ *
+ * Returns: %TRUE if the checkout was successfull, %FALSE otherwise.
+ *
+ **/
+gboolean
+ggit_repository_checkout_head (GgitRepository       *repository,
+                               GgitCheckoutOptions  *options,
+                               GError              **error)
+{
+	gint ret;
+
+	g_return_val_if_fail (GGIT_IS_REPOSITORY (repository), FALSE);
+	g_return_val_if_fail (GGIT_IS_CHECKOUT_OPTIONS (options), FALSE);
+	g_return_val_if_fail (error == NULL || *error == NULL, FALSE);
+
+	ret = git_checkout_head (_ggit_native_get (repository),
+	                         _ggit_checkout_options_get_checkout_options (options));
+
+	if (ret != GIT_OK)
+	{
+		_ggit_error_set (error, ret);
+		return FALSE;
+	}
+
+	return TRUE;
+}
+
+/**
+ * ggit_repository_checkout_index:
+ * @repository: a #GgitRepository.
+ * @index: (allow-none): a #GgitIndex or %NULL.
+ * @options: (allow-none): a #GgitCheckoutOptions or %NULL.
+ * @error: a #GError for error reporting or %NULL.
+ *
+ * Update files in the working tree to reflect the contents of the index. If
+ * @index is %NULL, then the current index of the repository will be used. If
+ * @options is %NULL, then the default checkout options will be used.
+ *
+ * If the checkout was not successfull, then @error will be set.
+ *
+ * Returns: %TRUE if the checkout was successfull, %FALSE otherwise.
+ *
+ **/
+gboolean
+ggit_repository_checkout_index (GgitRepository       *repository,
+                                GgitIndex            *index,
+                                GgitCheckoutOptions  *options,
+                                GError              **error)
+{
+	gint ret;
+
+	g_return_val_if_fail (GGIT_IS_REPOSITORY (repository), FALSE);
+	g_return_val_if_fail (index == NULL || GGIT_IS_INDEX (index), FALSE);
+	g_return_val_if_fail (GGIT_IS_CHECKOUT_OPTIONS (options), FALSE);
+	g_return_val_if_fail (error == NULL || *error == NULL, FALSE);
+
+	ret = git_checkout_index (_ggit_native_get (repository),
+	                          index != NULL ? _ggit_index_get_index (index) : NULL,
+	                         _ggit_checkout_options_get_checkout_options (options));
+
+	if (ret != GIT_OK)
+	{
+		_ggit_error_set (error, ret);
+		return FALSE;
+	}
+
+	return TRUE;
+}
+/**
+ * ggit_repository_checkout_tree:
+ * @repository: a #GgitRepository.
+ * @tree: (allow-none): a #GgitObject or %NULL.
+ * @options: (allow-none): a #GgitCheckoutOptions or %NULL.
+ * @error: a #GError for error reporting or %NULL.
+ *
+ * Update files in the working tree to reflect the contents of the specified
+ * commit, tag or tree object. If @tree is %NULL, then the current HEAD of the
+ * repository will be used. If @options is %NULL, then the default checkout
+ * options will be used.
+ *
+ * If the checkout was not successfull, then @error will be set.
+ *
+ * Returns: %TRUE if the checkout was successfull, %FALSE otherwise.
+ *
+ **/
+gboolean
+ggit_repository_checkout_tree (GgitRepository       *repository,
+                               GgitObject           *tree,
+                               GgitCheckoutOptions  *options,
+                               GError              **error)
+{
+	gint ret;
+
+	g_return_val_if_fail (GGIT_IS_REPOSITORY (repository), FALSE);
+	g_return_val_if_fail (tree == NULL || GGIT_IS_OBJECT (tree), FALSE);
+	g_return_val_if_fail (GGIT_IS_CHECKOUT_OPTIONS (options), FALSE);
+	g_return_val_if_fail (error == NULL || *error == NULL, FALSE);
+
+	ret = git_checkout_tree (_ggit_native_get (repository),
+	                         tree != NULL ? _ggit_native_get (tree) : NULL,
+	                         _ggit_checkout_options_get_checkout_options (options));
+
+	if (ret != GIT_OK)
+	{
+		_ggit_error_set (error, ret);
+		return FALSE;
+	}
+
+	return TRUE;
+}
+
+/**
+ * ggit_repository_revert:
+ * @repository: a #GgitRepository.
+ * @commit: a #GgitCommit.
+ * @options: a #GgitRevertOptions.
+ * @error: a #GError for error reporting, or %NULL.
+ *
+ * Revert the given @commit on top of the current working directory.
+ *
+ * Returns: %TRUE if the revert was successfull, %FALSE otherwise.
+ *
+ **/
+gboolean
+ggit_repository_revert (GgitRepository     *repository,
+                        GgitCommit         *commit,
+                        GgitRevertOptions  *options,
+                        GError            **error)
+{
+	gint ret;
+
+	g_return_val_if_fail (GGIT_IS_REPOSITORY (repository), FALSE);
+	g_return_val_if_fail (GGIT_IS_COMMIT (commit), FALSE);
+	g_return_val_if_fail (error == NULL || *error == NULL, FALSE);
+
+	ret = git_revert (_ggit_native_get (repository),
+	                  _ggit_native_get (commit),
+	                  options ? _ggit_revert_options_get_revert_options (options) : NULL);
+
+	if (ret != GIT_OK)
+	{
+		_ggit_error_set (error, ret);
+		return FALSE;
+	}
+
+	return TRUE;
+}
+
 /* ex:set ts=8 noet: */
